#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <signal.h>

#include <math.h>

#include "ff.h"
#include "fb.h"

#include "v1.h"
#include "v2.h"
#include "v3.h"
#include "v4.h"
#include "v5.h"
#include "v6.h"

static fftw_plan Plan;

static intype Buf[N];

double In[N];
double Power[N/2+1];

int Quittah=0;
static void QuitProc(int dummy)
{
	Quittah=dummy;
}

static char Cmd[256];
static int Heelp=0;
static void He(char **dummy)
{
	Heelp=1;
	dummy=dummy;
}
static void Ve(char **dummy)
{
	printf("Frequency Filter (ff) version 6.00\n");
	printf("Copyright (C) 1992,2000 Bisqwit (http://iki.fi/bisqwit/)\n");
	if(dummy)exit(0);
}

static struct
{
	void (*Func)(char **);
	char c, c2;
	const char *txt, *descr;
} Options[] =
{
	{He,'?',  0,"help",      "Help"},
	{Ve,'V',  0,"version",   "Version information"},
	{NULL,0,0,NULL,NULL}
};
static void ArgError(int a, const char *b)
{
	if(a)printf("ff: unrecognized option `--%s'", b);
	else
	{
		printf("ff: illegal option -- %c", *b);
		for(a=0; Options[a].c; a++)
			if(*b == Options[a].c)
			{
				putchar(b[1]);
				break;
			}
	}
	printf("\nTry `ff --help' for more information.\n");
	exit(1);
}
static void ReadArgs(int argc, char **argv)
{
	int a;

	for(a=1; a<argc; a++)
	{
		char *s = argv[a];

ReArg:	if(!s)continue;
		while(*s==' ' || *s=='\t')s++;
		if(!*s)continue;

		if(*s == '-' || (*s=='/'&&s[1]=='?'))
		{
			if(*++s == '-')
			{
				int b=0;
				for(s++; Options[b].c; b++)
					if(!strcmp(s, Options[b].txt))
					{
						s = argv[++a];
						Options[b].Func(&s);
						goto ReArg;
					}
				ArgError(1, s);
			}
			while(s && *s)
			{
				int b,c;
				for(b=c=0; Options[b].c; b++)
					if(s[0]==Options[b].c
					&&(s[1]==Options[b].c2
						  ||!Options[b].c2))
					{
						c=1;
						s++;
						if(Options[b].c2)s++;
						Options[b].Func(&s);
						s--;
					}
				if(!c)ArgError(0, s);
				if(s)s++;
			}
		}
		else
		{
			if(Cmd[0])
			{
				printf("ff: too many parameters -- `%s'\n", s);
				exit(EXIT_FAILURE);
			}
			strncpy(Cmd, s, 255);
		}
	}
	
	if(Heelp || !Cmd[0])
	{
		Ve(NULL);
		
		printf("\nUsage: ff [-");
		for(a=0; Options[a].c; a++)if(!Options[a].c2)putchar(Options[a].c);
		printf("] command-which-makes-the-input-in-mono-16-bit-%d-Hz-format\n\n",
			FREQ);

		for(a=0; Options[a].c; a++)
			printf("  -%c%c%c--%-14s%s\n",
				Options[a].c,
				Options[a].c2?Options[a].c2:',',
				Options[a].c2?',':' ',
				Options[a].txt,
				Options[a].descr);

		printf("\n");
		
		exit(EXIT_SUCCESS);
	}
}

int main(int argc, char **argv)
{
	int a;
	int fd;
	FILE *fp = NULL;
	
	time_t Begin;
	
	ReadArgs(argc, argv);
	
	printf("Cmd = '%s'\n", Cmd);	
	Trap((fp=popen(Cmd, "r"))==NULL?-1:1);
	
	fd = fileno(fp);
	
	{FILE *fp = fopen("ff.wisdom", "r");
	 if(fp){fftw_import_wisdom_from_file(fp);fclose(fp);}
	}

    double Out[N];

	Plan = fftw_plan_dft_r2c_1d(N, In, Out, FFTW_MEASURE);
	
	read(fd, &Buf[0], (N)*(sizeof(intype)));
#if VALDEBUG
	{int ValMin=inadd+inmax, ValMax=-inmax-inmax;long ValSum=0;
#endif	
	for(a=0; a<N; a++)
	{
#if VALDEBUG
		if(Buf[a] < ValMin)ValMin=Buf[a];
		if(Buf[a] > ValMax)ValMax=Buf[a];
		ValSum += Buf[a];
#endif
		In[a] = (Buf[a]+inadd)/(double)inmax;
	}
#if VALDEBUG
	printf("min=%d, max=%d, avg=%ld\n", ValMin, ValMax, ValSum/N);
	}return 0;
#endif

	signal(SIGINT, QuitProc);
	
	puts("Initializing dev");
	fb_init(&V6);
	puts("Done init");
	
	Begin = time(NULL);
	
	while(!Quittah)
	{
		time_t Now;
		
		fftw_execute(Plan);

		Power[0] = fabs(Out[0]);
		for(a=1; a < (N+1)/2; a++)Power[a] = log10(Out[a]*Out[a] + Out[N-a]*Out[N-a]);
		if(!(N%2))Power[N/2] = log10(Out[N/2]*Out[N/2]);
		
		Now = time(NULL);
		
		fb_view(&Begin, &Now);
		
		if(Step!=N)
		{
			memmove(&Buf[0], &Buf[Step], (sizeof(intype))    * (N-Step));
			memmove(&In[0],  &In[Step],  (sizeof(double)) * (N-Step));
		}
		
		/* Read more input data */
		if(!read(fd, &Buf[N-Step], (Step)*(sizeof(intype))))break;
		for(a=0; a<Step; a++)
		{
			In[N-Step+a] = (Buf[N-Step+a]+inadd)/(double)inmax;
		}
	}
	
	puts("Done loop");
	fb_done();
	puts("Done dev");
	
	fftw_destroy_plan(Plan);
	
	{FILE *fp = fopen("ff.wisdom", "w");
	 if(fp){fftw_export_wisdom_to_file(fp);fclose(fp);}
	}

	if(fp)
		pclose(fp);
	else
		close(fd);
		
	puts("End");
	
	return 0;
}
